-- 登录服务器列表数据
Lua_ServerListData = class("Lua_ServerListData");
function Lua_ServerListData:ctor()
    self:superCall("ctor");

    self.isBan = false;
    self.zoneList = Array();
    self.serverDic = {};
    self.allServerDic = {};
    self.curServerData = nil;
    self.localServerList = nil;
end

function Lua_ServerListData:fromServer(list)
    if list == nil or #list <= 0 then
        return;
    end
    self.localServerList = list;
end

function Lua_ServerListData:popErrorTipsWin()
    local tipsData = { des = LangDefine.getValue(16), btnType = 4, sureLabel = LangDefine.getValue(1) };
    Lua_WindowUtil.openWin("Lua_LoginTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
end

function Lua_ServerListData:readFromJson(param, bShowTips)
    for zone, arr in pairs(self.allServerDic) do
        arr:clear();
    end

    if bShowTips == nil then bShowTips = true; end
    if param.data ~= nil then param = param.data; end
    if param == nil or param.zone == nil or param.server == nil then
        if bShowTips then self:popErrorTipsWin(); end
        return false;
    end

    local zones = param.zone;
    local servers = param.server;
    local obj;
    local vo;
    local zoneStr;
    for i = 1, #servers do
        obj = servers[i];
        vo = self.serverDic[obj.flag];
        if vo == nil then
            vo = Lua_OneServerData.new();
            self.serverDic[obj.flag] = vo;
        end
        vo:setConfig(obj)

        zoneStr = tostring(vo.zone);
        vo.zoneName = tostring(zones[zoneStr]);

        if vo.isCreateRole then
            -- 已有角色服务器
            self:addServer(-1, obj.flag);
        elseif vo.ad_status == 1 or vo.ad_status == 2 then
            -- 流畅且没有角色，加入推荐列表
            self:addServer(-2, obj.flag);
        end
        self:addServer(vo.zone, obj.flag);
    end


    local recommendList = self:getServerListByZone(-2);
    local len = #zones + 1; -- 我的服务器
    if recommendList and recommendList:getSize() > 0 then -- 推荐服务器
        len = len + 1;
    end
    len = self.zoneList:getSize() - len;
    while (len > 0) do
        self.zoneList:remove(len);
        len = len - 1;
    end

    local index = 1;
    -- 推荐服务器
    if recommendList and recommendList:getSize() > 0 then -- 推荐服务器
        vo = self.zoneList[index];
        if vo == nil then
            vo = Lua_ServerButtinData.new();
            self.zoneList:add(vo);
        end
        vo.zone = -2;
        vo.name = LanUtil.getValue(123629);
        index = index + 1;
    end

    -- 我的服务器
    vo = self.zoneList[index];
    if vo == nil then
        vo = Lua_ServerButtinData.new();
        self.zoneList:add(vo);
    end
    vo.zone = -1;
    vo.name = LanUtil.getValue(123608);

    for k, v in pairs(zones) do
        index = index + 1;
        vo = self.zoneList[index];
        if vo == nil then
            vo = Lua_ServerButtinData.new();
            self.zoneList:add(vo);
        end
        vo.zone = tonumber(k);
        vo.name = tostring(v);
    end
    self.zoneList:sort("zone", false);

    local users = param.user;
    ServerConfig.userID = users.openId;
    PlayerPrefsUtil.instance:SetString(SetUpDefine.USERID, ServerConfig.userID);

    if users.package ~= nil then
        ServerConfig.market = tonumber(tostring(users.package));
        -- ServerConfig.ChannelID = tostring(users.package);
    end

    if users.channel ~= nil then
        ServerConfig.channel = tostring(users.channel);
    end

    self.isBan = false;
    if users.is_ban ~= nil and tonumber(tostring(users.is_ban)) == 1 then -- 是否封禁1：是，0：否
        self.isBan = true;
    end

    if not strIsNullOrEmpty(users.sdk_id) then
        SDKInfo.sdkUid = users.sdk_id;
    end

    -- PC相关
    local recordSign = PlayerPrefsUtil.instance:GetInt(SetUpDefine.RECORD_PHONE_SIGN, 1);
    if recordSign > 0 and param.pc_param ~= nil then
        self:parsePcParam(param.pc_param);
    end

    return true;
end

function Lua_ServerListData:addServer(zone, flag)
    if self.allServerDic[zone] == nil then
        self.allServerDic[zone] = Array();
    end
    self.allServerDic[zone]:add(flag);
end

-- 解析pc相关
function Lua_ServerListData:parsePcParam(dic)
    local expiresTime = nil;
    local ticket = nil;
    if dic ~= nil then
        if dic.expires_time ~= nil then
            expiresTime = tostring(dic.expires_time);
        end

        if dic.ticket ~= nil then
            ticket = tostring(dic.ticket);
        end
    end

    if not LanUtil.IsNullOrEmpty(expiresTime) then
        PlayerPrefsUtil.instance:SetString(SetUpDefine.WEB_EXPIRES_TIME_STR, expiresTime);
    else
        PlayerPrefsUtil.instance:DeleteKey(SetUpDefine.WEB_EXPIRES_TIME_STR);
    end

    if not LanUtil.IsNullOrEmpty(ticket) then
        PlayerPrefsUtil.instance:SetString(SetUpDefine.WEB_TICKET_STR, ticket);
    else
        PlayerPrefsUtil.instance:DeleteKey(SetUpDefine.WEB_TICKET_STR);
    end
end

function Lua_ServerListData:getServerListByZone(zone)
    return self.allServerDic[zone];
end

function Lua_ServerListData:getOneServerVo(flag)
    return self.serverDic[flag];
end

-- 无角色，推荐自动创角得服务器
function Lua_ServerListData:getAutoLoginServer()
    if PlayerData.instance:getPhpConfigNumberValue(PlayerData.instance.phpConfigKey, "autoCreateRole") <= 0 then
        return nil;
    end

    if self.isBan then
        return nil;
    end

    local myServerList = self:getServerListByZone(-1);
    if myServerList and myServerList:getSize() > 0 then
        return nil;
    end

    local dt = 0;
    local st = 0;
    local rs = nil;
    local defaultRs = nil;
    for zone, arr in pairs(self.allServerDic) do
        if zone ~= -1 and zone ~= -2 then
            for i = 1, arr:getSize() do
                local vo = self:getOneServerVo(arr[i]);
                if vo.is_create ~= 0 and vo.status ~= -1 and vo.status ~= 0 then
                    if vo.default == 1 then
                        if dt < vo.open_time then
                            dt = vo.open_time;
                            defaultRs = vo;
                        end
                    else
                        if st < vo.open_time then
                            st = vo.open_time;
                            rs = vo;
                        end
                    end
                end
            end
        end
    end

    if defaultRs ~= nil then
        return defaultRs;
    elseif rs ~= nil then
        return rs;
    end

    return nil;
end

function Lua_ServerListData:getDefaultServer()
    if self.localServerList ~= nil then
        return self.localServerList[1];
    end

    local myServerList = self:getServerListByZone(-1);
    if myServerList and myServerList:getSize() > 0 then
        self:onListSort(myServerList, "login_time", true);
        return self:getOneServerVo(myServerList[1]);
    end
    
    local dt = 0;
    local st = 0;
    local rs = nil;
    local defaultRs = nil;
    local firstRs = nil;
    local selectRs = nil;
    local curFlag = PlayerPrefsUtil.instance:GetString(DataCenter.loginData.serverFlag, "");
    for zone, arr in pairs(self.allServerDic) do
        if zone ~= -1 and zone ~= -2 then
            for i = 1, arr:getSize() do
                local vo = self:getOneServerVo(arr[i]);
                if vo.flag == curFlag then
                    selectRs = vo;
                    break;
                end

                if firstRs == nil then -- 防止一个都没有
                    firstRs = vo;
                end

                if vo.is_create ~= 0 and vo.status ~= -1 and vo.status ~= 0 then
                    if vo.default == 1 then
                        if dt <= vo.open_time then
                            dt = vo.open_time;
                            defaultRs = vo;
                        end
                    else
                        if st <= vo.open_time then
                            st = vo.open_time;
                            rs = vo;
                        end
                    end
                end
            end
        end
    end

    if selectRs ~= nil then
        return selectRs;
    elseif defaultRs ~= nil then
        return defaultRs;
    elseif rs ~= nil then
        return rs;
    elseif firstRs ~= nil then
        return firstRs;
    end

    return nil;
end

function Lua_ServerListData:onListSort(arr, key, up)
    if arr == nil then
        return;
    end
    
    local len = arr:getSize();
    for i = 1, len do
        for j = i + 1, len do
            local a = self:getOneServerVo(arr[i])[key];
            local b = self:getOneServerVo(arr[j])[key];
            local r = a < b;
            if up == false then
                r = not r;
            end
            if (r) then
                temp = arr[i];
                arr[i] = arr[j];
                arr[j] = temp;
            end
        end
    end
end

function Lua_ServerListData:dispose()
    if self.zoneList then
        self.zoneList:clear();
        self.zoneList = nil;
    end
    if self.serverDic then
        for k, _ in pairs(self.serverDic) do
            self.serverDic[k] = nil;
        end
        self.serverDic = nil;
    end
    if self.allServerDic then
        for k, v in pairs(self.allServerDic) do
            v:clear();
            self.allServerDic[k] = nil;
        end
        self.allServerDic = nil;
    end
    self.curServerData = nil;
    self.localServerList = nil;
end










Lua_ServerButtinData = class("Lua_ServerButtinData")
function Lua_ServerButtinData:ctor()
    self:superCall("ctor");

    self.zone = 0;
    self.name = nil;
end