-- sdk接口
checkRequire("Lua_SDKMessageType");
local SDKInterface = SDKInterface;
Lua_SDKInterface = class("Lua_SDKInterface");
Lua_SDKInterface.TYPE_TABLESTR = "table";
function Lua_SDKInterface:ctor()
	self:superCall("ctor");
	self:initEvents();
end

function Lua_SDKInterface:dispose()
	self.pointOutFormData = nil;
	self.login_token = nil;
end

function Lua_SDKInterface:initEvents()
	MsgUtil.addEventListener(EventsDefine.SDK_MESSAGE_UPDATE, self, self.onSDKMessage); -- sdk通知
end

-- 调用sdk接口, dic为table
function Lua_SDKInterface:callSDKInterfaceTab(methobName, param)
	local paramJsonStr = nil;
	if param ~= nil then paramJsonStr = decodeTabToJson(param); end
	self:callSDKInterface(methobName, paramJsonStr)
end

-- 调用sdk接口, paramJsonStr为json格式
function Lua_SDKInterface:callSDKInterface(methobName, paramJsonStr)
	SDKInterface.CallSDKInterface(methobName, paramJsonStr);
end

-- 记录到bugly的日志
function Lua_SDKInterface:log(logStr)
	self:callBuglyInterface(Lua_SDKMessageType.bugly_buglylog, 0, "Log", logStr);
end

-- 调用bugly接口
function Lua_SDKInterface:callBuglyInterface(methobName, param1, param2, param3, param4)
	if param1 ~= nil and type(param1) == Lua_SDKInterface.TYPE_TABLESTR then param1 = decodeTabToJson(param1); end
	if param2 ~= nil and type(param2) == Lua_SDKInterface.TYPE_TABLESTR then param2 = decodeTabToJson(param2); end
	if param3 ~= nil and type(param3) == Lua_SDKInterface.TYPE_TABLESTR then param3 = decodeTabToJson(param3); end
	if param4 ~= nil and type(param4) == Lua_SDKInterface.TYPE_TABLESTR then param4 = decodeTabToJson(param4); end
	SDKInterface.CallBuglyInterface(methobName, param1, param2, param3, param4)
end

-- sdk 通知，json格式
-- {"func":"标识","param":"参数集合"}
function Lua_SDKInterface:onSDKMessage(msg)
	LogManager.showLog("Lua_SDKInterface onSDKMessage : "..tostring(msg));
	if msg == nil or msg == "" then
		return;
	end

	local dic = decodeJson(msg);
    if dic == nil then
    	return;
    end

    local func = dic.func;
    if func == Lua_SDKMessageType.OrderPefixExt then -- 下单额外参数返回
    	-- print("order_pefix_ext: ", dic.param);
    	if self.pointOutFormData ~= nil then
    		self.pointOutFormData.order_pefix_ext = dic.param;
    		self:PointOutFromData(self.pointOutFormData);
    		self.pointOutFormData = nil;
    	end
	else
	    local param = dic.param ~= nil and decodeJson(dic.param) or {};
	    if func == Lua_SDKMessageType.netState then -- 网络
	    	SDKInfo.NetState = param.netState;
	        if param.netState == Lua_SDKMessageType.NO_NET and GlobalVar.isInGame == 1 then
	            GlobalVar.canReconnect = true;
	            DataCenter.connectionManager:errorAlert(LanUtil.getValue(10003), true);
	        end
	    elseif func == Lua_SDKMessageType.phoneinfo then -- 设备信息
	    	self:setPhoneInfo(param);
	    elseif func == Lua_SDKMessageType.login then -- 登陆成功返回给客户端
	    	if param.uid ~= nil then
	    		SDKInfo.sdkUid = param.uid;
	    	elseif param.sdkUid ~= nil then
	    		SDKInfo.sdkUid = param.sdkUid;
	    	end
	    	DataCenter.gameReported:loginReportedData(Lua_GameReported.REPORT_SDK_LOGIN_SUCCESS);
	    	self.login_token = decodeTabToJson(param);
	    	Lua_MsgUtil.callFunc("LOGIN_BACK_SEVER_SELECTED");
	    elseif func == Lua_SDKMessageType.changeuser then -- 切换用户成功返回给客户端
		    if param.uid ~= nil then
	    		SDKInfo.sdkUid = param.uid;
	    	elseif param.sdkUid ~= nil then
	    		SDKInfo.sdkUid = param.sdkUid;
	    	end
		    DataCenter.gameReported:loginReportedData(Lua_GameReported.REPORT_SDK_CHANGEUSER);
	    	self.login_token = decodeTabToJson(param);
	        DataCenter.connectionManager:loginOut();
	    elseif func == Lua_SDKMessageType.logout then -- 注销
	    	DataCenter.gameReported:loginReportedData(Lua_GameReported.REPORT_SDK_LOGOUT);
	    	self.login_token = nil;
	    	SDKInfo.sdkUid = nil;
	    	ServerConfig.userID = nil;
	    	DataCenter.connectionManager:loginOut();
	    elseif func == Lua_SDKMessageType.Openalbum then -- 上传图片
	    	Lua_MsgUtil.callFunc("SELECT_FILE", param.imagepath);
	    elseif func == Lua_SDKMessageType.ScreenHeight then -- 刘海高度
	    	ServerConfig.sdkLiuhaiHight = param.ScreenHeight;
	    elseif func == Lua_SDKMessageType.RunBackGround then --
	    	local runState = (param.runState ~= nil) and tonumber(tostring(param.runState)) or 0;
	    	Lua_MsgUtil.callFunc("STATE_RUNBACKGROUND_CHANGE", runState);
	    end
	end
end

-- 设备信息
function Lua_SDKInterface:setPhoneInfo(param)
	if param == nil then return; end
	SDKInfo.MACAddress = param.mac;
	SDKInfo.PhoneModel = param.sys_model;
	SDKInfo.SystemVersion = param.sys_version;
	SDKInfo.AvailMemory = param.avail_memory;
	SDKInfo.CustomDevice = param.custom_device;
	SDKInfo.OpenGL = param.openGl;
	SDKInfo.group_id = param.group_id;
	SDKInfo.Paltform = param.platform_tag;
	SDKInfo.ChannelID = param.channel_id;
	SDKInfo.sub_channel_id = param.sub_channel_id;
	SDKInfo.brand = param.brand;
	SDKInfo.device_model = param.device_model;
	SDKInfo.os_version = param.os_version;
	SDKInfo.udid = param.udid;
	SDKInfo.Imei = param.imei;
	SDKInfo.Idfa = param.idfa;
	SDKInfo.oaid = param.oaid;
	SDKInfo.caid = param.caid;
	SDKInfo.android_id = param.android_id;
	SDKInfo.app_channel = param.app_channel;
	-- param.screen_length
	-- param.screen_width
	SDKInfo.NetState = param.network_type;
	SDKInfo.app_version = param.app_version;
	SDKInfo.appkey = param.appkey;
	SDKInfo.game_tag = param.game_tag;
	SDKInfo.is_emulator = param.is_emulator == "true";
	SDKInfo.timezone_offset = param.timezone_offset ~= nil and tonumber(tostring(param.timezone_offset)) or 0;
	DataCenter.gameReported:loginReportedData(Lua_GameReported.REPORT_PHONE_DEVICE_INFO);
	SDKInterface.Instance:uploadChannelActivationPost();
end

function Lua_SDKInterface:PointOutById(goldNum, rechargeId)
	local config = Lua_RechargeConfigData.getRechargeConfig(rechargeId);
	if config == nil then
		return;
	end
	self:PointOut(goldNum, tonumber(config.money), tonumber(config.money_type), tonumber(rechargeId), config.gift_name, config.gold);
end

-- conf_id:1正常
-- money_type 1：人民币 2： 美金 3： 台币 4： 韩币 5： 越南币 6 ：新加坡币 7 ：泰株
function Lua_SDKInterface:PointOut(goldNum, money, money_type, conf_id, goodsName, diamondNum, useTicket)
	if self.pointOutFormData ~= nil then return; end -- 上一个支付还未完成则不进行下一个
	if money_type == nil then money_type = 1; end
	if useTicket == nil then useTicket = true; end
	local vo = Lua_RechargeConfigData.getRechargeConfig(conf_id);

	if useTicket then
		local ticketId = DataCenter.packUtil:getTicketId();
		local ticketAmount = DataCenter.packData:getItemCountById(ticketId);
		if ticketAmount ~= nil and ticketAmount > 0 then -- 判断代金券数量
			local convert_ratio = getConfig("item", ticketId, "convert_ratio");
			if convert_ratio == nil or convert_ratio <= 0 then
				convert_ratio = 1;
			end
			ticketAmount = ticketAmount * convert_ratio;
			if ticketAmount >= vo.money then
				local tipsData = {
					des = LanUtil.formateStr(130000, spliceIconStrById(ticketId), DataCenter.packData:getItemCountById(ticketId), vo.money),
					btnType = 1,
					owner = self,
					cancelFun = self.ticketCancelFun,
					sureFun = self.ticketSureFun,

					goldNum = goldNum,
					money = money,
					money_type = money_type,
					conf_id = conf_id,
					goodsName = goodsName,
					diamondNum = diamondNum,
				};
				Lua_WindowUtil.openWin("Lua_TipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
				return;
			end
		end
	end

	if strIsNullOrEmpty(goodsName) or diamondNum == nil then
		goodsName = vo.gift_name;
		diamondNum = vo.gold;
	end

    local formData = {
    	getOrderId = 1,
    	server = ServerConfig.sign,
    	package = tostring(ServerConfig.market),
    	role_id = DataCenter.roleData.roleId,
    	money = money,
    	money_type = money_type,
    	account = ServerConfig.userID,
    	conf_id = conf_id,
    	role_level = DataCenter.roleData.level,
    	vip_level = DataCenter.roleData.vip,
    	mac = ServerConfig.mac
    };
	if Lua_PlatformUtil.IsAndroid() then
		formData.imei = SDKInfo.Imei;
	elseif Lua_PlatformUtil.IsIPhonePlayer() then
		formData.idfa = SDKInfo.Idfa;
    end

    -- 获取订单号
    -- local tokenstr = decodeTabToJson(formData);
    formData.goodsName = goodsName;
    formData.diamondNum = diamondNum;
    formData.orderType = vo.type;
    -- local extParam = decodeTabToJson(formData);
    -- UploadManager.getInstance():uploadPhpPost(UrlManager.prefixRequestStr, tokenstr, extParam, self, self.loadServerComplete, self.loadServerError, true);

    if Lua_PlatformUtil.IsNoSdk() then
	    self:PointOutFromData(formData);
	else
		self.pointOutFormData = formData;
		self:callSDKInterface(Lua_SDKMessageType.GetOrderPefixExt);
	end
end

function Lua_SDKInterface:PointOutFromData(formData)
	if formData == nil then
		return;
	end

	-- 获取订单号
    local extParam = decodeTabToJson(formData);
    formData.goodsName = nil;
    formData.diamondNum = nil;
    formData.orderType = nil;
    local tokenstr = decodeTabToJson(formData);
    UploadManager.getInstance():uploadPhpPost(UrlManager.prefixRequestStr, tokenstr, extParam, self, self.loadServerComplete, self.loadServerError, true);
end

function Lua_SDKInterface:ticketCancelFun(data)
	self:PointOut(data.goldNum, data.money, data.money_type, data.conf_id, data.goodsName, data.diamondNum, false)
end

function Lua_SDKInterface:ticketSureFun(data)
	local arr = Array();
	arr:add(data.conf_id);

	local ticketId = DataCenter.packUtil:getTicketId();
	local ticketVo = DataCenter.packData:getItemVoById(ticketId);
	Lua_ProxyManager.getProxy("Lua_PackProxy"):sUseItem(ticketVo.pack_type, ticketVo.unique_id, 1, arr);
end

function Lua_SDKInterface:loadServerError(text, extParam)
    print("请求订单错误--", text, extParam);
end

function Lua_SDKInterface:loadServerComplete(text, extParam)
	-- print("Lua_SDKInterface PointOut text : ", text, "extParam : ", extParam);
	if strIsNullOrEmpty(text) or strIsNullOrEmpty(extParam) then
		print("请求订单错误 text = nil", extParam);
		OperateAlert.show(LanUtil.formateStr(107708, "text=nil"));
		return;
	end

	local dictionary = decodeJson(text);
	if dictionary == nil then
		print("请求订单号text Json解析失败", text, extParam);
		OperateAlert.show(LanUtil.formateStr(107708, LanUtil.getValue(107709)));
		return;
	end

	local error = dictionary.error;
    if not strIsNullOrEmpty(error) then
        print("请求订单号失败：", error, text, extParam);
        OperateAlert.show(LanUtil.formateStr(107708, "error:" .. error));
        return;
    end

    local extParamDic = decodeJson(extParam);
    if extParamDic == nil then
    	print("请求订单号extParam Json解析失败", text, extParam);
    	return;
    end
    
	local data = dictionary;
	if dictionary.data ~= nil then
		data = dictionary.data;
	end

    if Lua_PlatformUtil.IsAndroid() or Lua_PlatformUtil.IsIPhonePlayer() then
		if Lua_PlatformUtil.IsNoSdk() then
			self:testEditorRecharge(data.orderId, extParamDic.money);
			return;
		end

		local formData = {};
		formData.configId = extParamDic.conf_id;
		formData.orderType = extParamDic.orderType;
    	formData.goodsName = extParamDic.goodsName;
    	formData.goodsNum = extParamDic.diamondNum;
    	formData.money = extParamDic.money;
    	formData.myParam = dictionary;
		self:callSDKInterface(Lua_SDKMessageType.PointOut, decodeTabToJson(formData));
	else
		local linkUrl = data.links;
        if not strIsNullOrEmpty(linkUrl) then -- 跳转
            UnityEngine.Application.OpenURL(linkUrl);
            return;
        end
        self:testEditorRecharge(data.orderId, extParamDic.money);
    end
end

function Lua_SDKInterface:testEditorRecharge(orderId, rechargeNum)
    local url = UrlManager.FIX_HTTP .. ServerConfig.center_server .. "/single/charge/package/" .. ServerConfig.market .. "?extends=" .. orderId .. "|" .. ServerConfig.sign;
    LoadManager.getInstance():Load(Lua_LoadFileType.JSON, url, true);
    print("testEditorRecharge: ", url);
end